#!/usr/bin/perl -w
use strict;
use DBI;
use Getopt::Long;

my $NAME   = '';
my $RESTORE= 0;
my $LIST   = 0;
my $VCF    = '';
my $DB     = '';
my $HOST   = 'localhost';
my $USER   = '';
my $PASS   = '';

################################################################
# Checking parameters and database connection
################################################################
GetOptions( 'n|name=s'      => \$NAME,
            'r|restore'     => \$RESTORE,
            'l|list'        => \$LIST,
            'f|file=s'      => \$VCF,
            'db|database=s' => \$DB,
            'h|host=s'      => \$HOST,
            'u|user=s'      => \$USER,
            'p|password=s'  => \$PASS
);

if ($DB eq ''||($RESTORE&&!$NAME)||(!$RESTORE&&!-e$VCF)) {
  _error_method();  
  exit;
}

my $connect = DBI->connect('DBI:mysql:database='.$DB.';host='.$HOST.';mysql_local_infile=1;',$USER,$PASS)
  or die "Can't connect to database: $DBI::errstr\n";

###########################################################
# Checking base table
###########################################################
if ( ! defined _check_exists_table("sys_accession") ) {
  print "Some basic table does not exists.\n";
  print "Did you excute 'tasuke_init.pl' at the first?\n";
  exit;
}

###########################################################
# Listing track
###########################################################
if ($LIST) {
  print "[List of accessions] :\n";
  foreach my $l (_lists()) {
    print "$l\n";
  }
  exit;
}

###########################################################
# Checking accession id
###########################################################
my $TABLE =_check_db();
if ( ! defined $TABLE ) {
  print "'$NAME' is not exists on list.\n\n";
  print "- Here is list you loaded\n";
  foreach my $l (_lists()) {
    print "$l\n";
  }
  exit;
}
$TABLE.="_accession";

###########################################################
# Restoring depth table
###########################################################
if ($RESTORE) {
  _dbexe("TRUNCATE TABLE $TABLE;");
  print "DEPTH $NAME table is restored.\n";
  exit;
}

###########################################################
#  Checking DEPTH file
###########################################################
 my $num=0;
 _print_time("File checking");
 open IN, '<', $VCF or die "file open error: $!";
 while (<IN>) {
 	
	if($_ !~ /^postion/){
		$num++;
		my @line = split(/t/, $_);
		foreach my $x(@line){
			if($x eq ""){
				print "DEPTH file is probably truncated. near:$num \n";
				print $_."\n";
				print "Exit.\n";
				exit;
			}
		}
	}
 }
 my $chk_max = _max();
 if($chk_max != $num){
	print "DEPTH file is probably truncated. near:$num\n";
	print "Exit.\n";
	exit;
 }
 
###########################################################
# Creating depth table
###########################################################

  # Set chromosomes 
  my (@chrs) = _chrs();
  my ($max) = _max();

  _dbexe("LOCK TABLES $TABLE WRITE, sys_chromosome READ;");
  _print_time(" [".$NAME." depth] begin create table");
  _LoadDepth($VCF, $TABLE, $NAME);
  _dbexe("UNLOCK TABLES;");
  _print_time(" [".$NAME." depth] complete create table");

$connect->disconnect or warn "Disconnection error: $DBI::errstr\n";
exit;



### Checking id on the list ####################################
sub _check_db {
  my $val = $connect->selectrow_array("select accession_db from sys_accession where accession_id='$NAME'");
  return $val;
}

### Execute DB ################################################
sub _dbexe {
  my ($query) = @_;
  my $query_handle = $connect->prepare($query);
  $query_handle->execute() or warn "Excute error: $DBI::errstr\n";
}

sub _error_method {
  print "\nCreating depth db\n";
  print "tasuke_depth_db.pl -n [id] -f [depth_file] -db [database] -u [user] -p [password]\n\n";
  print "Restoring table\n";
  print "tasuke_depth_db.pl -r -n [name] -db [database] -u [user] -p [password]\n\n";
  print "Listing accession\n";
  print "tasuke_depth_db.pl -l -db [database] -u [user] -p [password]\n\n";
}

########LOAD DEPTH DATA

sub _LoadDepth{
	my $DEPTH = shift;
	my $TABLE = shift;
	my $NAME = shift;
	
	#_print_time(" [".$NAME." depth] begin update table");
	_dbexe("LOAD DATA LOCAL INFILE "."\'".$DEPTH."\'"." replace INTO TABLE ".$TABLE." IGNORE 1 LINES ;");
	#_print_time(" [".$NAME." depth] complete update table");
	
}

# max
sub _max {
  my $val;
  my $query = "SELECT max(size) FROM sys_chromosome";
  my $query_handle = $connect->prepare($query);
  $query_handle->execute();
  $val = $query_handle->fetchrow_hashref;
  return $val->{'max(size)'};
}

# chromosomes
sub _chrs {
  my $val;
  my @c;
  my $query = "SELECT id FROM sys_chromosome";
  my $query_handle = $connect->prepare($query);
  $query_handle->execute();
  $query_handle->bind_columns(undef, \$val);
  while($query_handle->fetch()) {
    push @c, $val;
  }
  return @c;
}

sub _print_time {
  my ($str) = @_;
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
  $year += 1900;
  $mon += 1;
  my $datetime = sprintf "%04d-%02d-%02d %02d:%02d:%02d", $year, $mon, $mday, $hour, $min, $sec;
  print "$str : $datetime\n";
}

# Accession list
sub _lists {
  my $val;
  my @lists;
  my $query = "SELECT accession_id FROM sys_accession";
  my $query_handle = $connect->prepare($query);
  $query_handle->execute();
  $query_handle->bind_columns(undef, \$val);
  while($query_handle->fetch()) {
    push @lists, $val;
  }
  return @lists;
}

### Convert B to KB ###########################################


### Checking table ####################################
sub _check_exists_table {
  my ($t) = @_;
  my $val = $connect->selectrow_array("show tables like '$t'");
  return $val;
}

